/***************************************************************
  King's Korner for Gameboy
****************************************************************
  Written by Harold Toler (herg@bigfoot.com)
  August 28, 2000
****************************************************************
  Please email me before using this code in a project.  I'd
  not only like to know who's using it and for what,  but I'd
  also like to see your finished product.
  
  I'm sure the code isn't the best, but it works.  If you
  notice something I should do differently, let me know.
***************************************************************/

#include <rand.h>
#include <types.h>
#include <stdio.h>
#include <gb\gb.h>
#include <gb\cgb.h>

#include "backgrnd.h"
#include "backgrnd.c"  //data for background, generated by GBTD

#include "cardback.c"  //data for cards, generated by GBTD
#include "hole.c"
#include "trapcard.c"
#include "suits.c"
#include "middles.c"
#include "numbers.c"

#include "sprites.c"  //data for pointer and dot, generated by GBTD

#include "maps.c"     //maps for the cards and background

#include "start.c"

#include "sign.h"
#include "sign.c"
#include "signmap.c"

#include "color.c"     //color palettes

#define SPADES   0
#define CLUBS    1
#define HEARTS   2
#define DIAMONDS 3

#define HOLE 1
#define BACK 2

#define SUIT_MASK       3
#define RANK_MASK       60

/***************************************************************
  Function prototypes
***************************************************************/
void title_screen(void);
void init_graphics(void);
void draw_background(void);
void draw_card(UBYTE x,UBYTE y,UBYTE rank,UBYTE suit);
void refresh_cards(void);
void place_pointer(UBYTE x,UBYTE y);
void place_dot(UBYTE x,UBYTE y);
void process_click(void);
UBYTE get_suit(UBYTE card);
UBYTE get_rank(UBYTE card);
void deal_cards(void);
void move_deck_discard(void);
void move_discard_tableau(UBYTE hole);
void check_joypad(void);
void win_animation(void);

/***************************************************************
  Global variables
***************************************************************/
UBYTE pointer_x,pointer_y;
UBYTE discard_active;
UBYTE tableau_active;

UBYTE deck[52];
UBYTE tableau[16];
UBYTE discard;

UBYTE deckindex;

UBYTE blank_tile[] = {0,0};

UWORD seed;

/***************************************************************
  Data used to draw the sprites
***************************************************************/
unsigned char sprites_tiles[] =
{
  0,1,2,3,4,5,6,7,8,9,10,11,12
};

/***************************************************************
  Draws the title screen
***************************************************************/
void title_screen(void)
{
  UWORD i,j,k,x,y,card,offset;
  
  SHOW_BKG;
  SHOW_SPRITES;

  offset=150;
    
  k=0;
  while(!((joypad())&(J_START)))
  {
    k++;
    delay(3);
    if(k<500)
    {
      do{card=rand();}while(card>51);
      do{x=rand();}while(x>20);
      do{y=rand();}while(y>18);
      if(_cpu == CGB_TYPE)
      { 
        VBK_REG = 1;
        if(card&2) set_bkg_tiles(x,y,2,3,literedcard_color_map);
        else set_bkg_tiles(x,y,2,3,liteblackcard_color_map);
        VBK_REG = 0; 
      }
      set_bkg_tiles(x,y,2,3,card_maps[card+3]);
    }
    else if(k==700)
    {
      if(_cpu == CGB_TYPE)
      { 
        VBK_REG = 1;
        for(i=2;i<=18;i++) 
          for(j=2;j<=7;j++)
            set_bkg_tiles(i,j,1,1,sign_color_map);
        VBK_REG = 0; 
      }
      set_bkg_tiles(2,2,16,5,sign_tiles);
    }
    else if(k>1000 && offset!=0)
    {
      for(i=0;i<10;i++) move_sprite(i+3,49+i*8,140+(int)(offset/6));
      if(offset!=0) offset--;
    }
    else if(offset==0)
    {
      if(k&128) for(i=0;i<10;i++) move_sprite(i+3,49+i*8,210);
      else for(i=0;i<10;i++) move_sprite(i+3,49+i*8,140);
    }
  }
  waitpadup();
  for(i=0;i<10;i++)
    move_sprite(i+3,40+i*8,190);
  seed = DIV_REG;

  pointer_x=20;
  pointer_y=20;
}

/***************************************************************
  Sets up all the graphics
***************************************************************/
void init_graphics(void)
{
  UBYTE i;

  DISPLAY_OFF;
  set_bkg_data(1,6,cardback_data);
  set_bkg_data(7,6,hole_data);
  set_bkg_data(13,2,trapcard_data);
  set_bkg_data(15,8,suits_data);
  set_bkg_data(23,2,middles_data);
  set_bkg_data(25,52,numbers_data);
  set_bkg_data(77,1,backgrnd_data);
  set_bkg_data(78,48,sign_data);
  set_sprite_data(0,3,sprites_data);
  set_sprite_data(3,10,start_data);

  for(i=0;i<13;i++)
    set_sprite_tile(i,sprites_tiles[i]);
  
  if(_cpu == CGB_TYPE)
  { 
    set_bkg_palette(0,7,&backgroundpalette[0]); 
    set_sprite_palette(0,2,&spritepalette[0]);
    set_sprite_prop(0,0);
    set_sprite_prop(1,0);
    set_sprite_prop(2,1);
    for(i=3;i<=12;i++) set_sprite_prop(i,0);
  } 

  DISPLAY_ON;
}

/***************************************************************
  Draws the background
***************************************************************/
void draw_background(void)
{
  UBYTE i,j;
  
  if(_cpu == CGB_TYPE)
  { 
    VBK_REG = 1;
    for(i=0;i<20;i++)
      for(j=0;j<18;j++)
        set_bkg_tiles(i,j,1,1,background_color_map);
    VBK_REG = 0; 
  }
  for(i=0;i<20;i++)
    for(j=0;j<18;j++)
      set_bkg_tiles(i,j,1,1,background_tiles);
}

/***************************************************************
  Draw a card on the screen
***************************************************************/
void draw_card(UBYTE x,UBYTE y,UBYTE rank,UBYTE suit)
{
  UBYTE index;
  
  index=4*rank+suit-1;

  if(_cpu == CGB_TYPE)
  { 
    VBK_REG = 1;
    if(rank==0 && suit==BACK) set_bkg_tiles(x,y,2,3,redcard_color_map);
    else if(rank==0 && suit==HOLE) set_bkg_tiles(x,y,2,3,blackcard_color_map);
    else if(suit&2) set_bkg_tiles(x,y,2,3,redcard_color_map);
    else set_bkg_tiles(x,y,2,3,blackcard_color_map);
    VBK_REG = 0; 
  }
  set_bkg_tiles(x,y,2,3,card_maps[index]);
}

/***************************************************************
  Redraw all the cards on the screen
***************************************************************/
void refresh_cards(void)
{
  UBYTE i;
  
  //Draw stock
  if(deckindex==0) draw_card(1,1,0,HOLE);
  else draw_card(1,1,0,BACK);
  
  //Draw discard
  if(get_rank(discard)==0) draw_card(4,1,0,HOLE);
  else draw_card(4,1,get_rank(discard),get_suit(discard));
  
  //Draw tableau
  for(i=0;i<16;i++)
  {
    if(tableau[i]==0) draw_card(8+(i&3)*3,1+((i&12)>>2)*4,0,HOLE);
    else 
    {
      draw_card(8+(i&3)*3,1+((i&12)>>2)*4,get_rank(tableau[i]),get_suit(tableau[i]));
    }
  }
}

/***************************************************************
  Moves the pointer
***************************************************************/
void place_pointer(UBYTE x, UBYTE y)
{
  move_sprite(0,x+8,y+16);
  move_sprite(1,x+8,y+16+8);
}

/***************************************************************
  Moves the dot
***************************************************************/
void place_dot(UBYTE x, UBYTE y)
{
  move_sprite(2,x+4,y+12);
}

/***************************************************************
  Determines what to do when the A button is pressed
***************************************************************/
void process_click(void)
{
  UBYTE clicked_column,clicked_row,clicked_hole,i,j;
  
  waitpadup();
  
  clicked_column=0;
  clicked_row=0;
  clicked_hole=0;
  
  if(pointer_x>=8*8 && pointer_x<=10*8) clicked_column=1;
  else if(pointer_x>=11*8 && pointer_x<=13*8) clicked_column=2;
  else if(pointer_x>=14*8 && pointer_x<=16*8) clicked_column=3;
  else if(pointer_x>=17*8 && pointer_x<=19*8) clicked_column=4;
  
  if(pointer_y>=1*8 && pointer_y<=4*8) clicked_row=1;
  else if(pointer_y>=5*8 && pointer_y<=8*8) clicked_row=2;
  else if(pointer_y>=9*8 && pointer_y<=12*8) clicked_row=3;
  else if(pointer_y>=13*8 && pointer_y<=16*8) clicked_row=4;
  
  if(clicked_column*clicked_row!=0)
  {
    clicked_hole=(clicked_column)+(clicked_row-1)*4;
    if(discard!=0 && tableau[clicked_hole-1]==0)
    {
      if(get_rank(discard)==11)
      {
        if(clicked_hole==5 || clicked_hole==8 || clicked_hole==9 || clicked_hole==12)
        {
          move_discard_tableau(clicked_hole-1);
        }
      }
      else if(get_rank(discard)==12)
      {
        if(clicked_hole==2 || clicked_hole==3 || clicked_hole==14 || clicked_hole==15)
        {
          move_discard_tableau(clicked_hole-1);
        }
      }
      else if(get_rank(discard)==13)
      {
        if(clicked_hole==1 || clicked_hole==4 || clicked_hole==13 || clicked_hole==16)
        {
          move_discard_tableau(clicked_hole-1);
        }
      }
      else move_discard_tableau(clicked_hole-1);
    }
    else if(get_rank(tableau[clicked_hole-1])==10) tableau[clicked_hole-1]=0;
    else if(tableau_active)
    {
      if(get_rank(tableau[clicked_hole-1])+get_rank(tableau[tableau_active-1])==10 
        && clicked_hole!=tableau_active)
      {
        tableau[clicked_hole-1]=0;
        tableau[tableau_active-1]=0;
        refresh_cards();
      }
      tableau_active=0;
      place_dot(200,200);
    }
    else if(tableau[clicked_hole-1]!=0 && ((!discard_active)||(discard==0)))
    {
      tableau_active=clicked_hole;
      place_dot(pointer_x,pointer_y);
    }
  }
  else if(pointer_x>=4*8 && pointer_x<=6*8 && pointer_y>=1*8 && pointer_y<=4*8)
  {
    j=0;
    for(i=0;i<16;i++) if(tableau[i]==0) j++;
    if(j!=0) discard_active=TRUE;
  }
  else
  {
    tableau_active=0;
    place_dot(200,200);
  }

  refresh_cards();
}

/*******************************************************
  Get the properties of a card
*******************************************************/
UBYTE get_suit(UBYTE card)
{
  return (card & SUIT_MASK);
}

UBYTE get_rank(UBYTE card)
{
  return ((card & RANK_MASK)>>2);
}

/*******************************************************
  Deal the cards
*******************************************************/
void deal_cards(void)
{
  UBYTE i,card;
  UBYTE usedcard[52];
    
  seed = (seed<<8) | DIV_REG; //this was nice place to put this

  initrand(seed);
  discard_active=FALSE;
  tableau_active=0;

  for(i=0;i<52;i++) deck[i]=0;
  for(i=0;i<16;i++) tableau[i]=0;
  discard=0;

  deckindex=0;

  for(i=0;i<52;i++) usedcard[i]=FALSE;

  for(i=0;i<52;i++)
  {
    do
    {
      do{card=rand();}while(card>51);
    }while(card>51||usedcard[card]);
    usedcard[card]=TRUE;
    do
    {
      do{deckindex=rand();}while(deckindex>51);
    }while(get_rank(deck[deckindex])!=0);
    deck[deckindex]=card+4;  //sets rank, suit in one pop
  }
  
  deckindex=52;
}

/*******************************************************
  Move a card from the deck to the discard
*******************************************************/
void move_deck_discard(void)
{
  deckindex--;
  discard=deck[deckindex];
  deck[deckindex]=0;
}

/*******************************************************
  Move a card from the deck to the discard
*******************************************************/
void move_discard_tableau(UBYTE hole)
{
  tableau[hole]=discard;
  discard=0;
}

/*******************************************************
  Check for a button press
*******************************************************/
void check_joypad(void)
{
  UBYTE i,j,button_temp;
  
  button_temp=joypad();
  if((button_temp & J_SELECT) && (button_temp & J_DOWN))
  {
    draw_background();
    deal_cards();
    refresh_cards();
  }
  if((button_temp & J_UP) && pointer_y!=0)
  {
    pointer_y--;
    place_pointer(pointer_x,pointer_y);
  }
  if((button_temp & J_DOWN) && pointer_y!=142)
  {
    pointer_y++;
    place_pointer(pointer_x,pointer_y);
  }
  if((button_temp & J_LEFT) && pointer_x!=0)
  {
    pointer_x--;
    place_pointer(pointer_x,pointer_y);
  }
  if((button_temp & J_RIGHT) && pointer_x!=158)
  {
    pointer_x++;
    place_pointer(pointer_x,pointer_y);
  }
  else if(button_temp & J_A) process_click();
  else if(button_temp & J_B)
  {
    waitpadup();

    tableau_active=0;
    place_dot(200,200);
    
    j=0;
    for(i=0;i<16;i++) if(tableau[i]==0) j++;
    if(j!=0) discard_active=TRUE;  //if there are holes in tableau, start dealing cards
  }
}

/***************************************************************
  Throws some cards around when you win
***************************************************************/
void win_animation(void)
{
  UBYTE x,y,card;
  
  waitpadup();
  while(!(joypad()))
  {
    do{card=rand();}while(card>51);
    do{x=rand();}while(x>20);
    do{y=rand();}while(y>18);
    if(_cpu == CGB_TYPE)
    { 
      VBK_REG = 1;
      if(card&2) set_bkg_tiles(x,y,2,3,redcard_color_map);
      else set_bkg_tiles(x,y,2,3,blackcard_color_map);
      VBK_REG = 0; 
    }
    set_bkg_tiles(x,y,2,3,card_maps[card+3]);
  }
  draw_background();
  deal_cards();
  refresh_cards();
}

/***************************************************************
  Main program
***************************************************************/
void main()
{
  UBYTE i,j;
  
  init_graphics();
  title_screen();
  draw_background();
  deal_cards();
  refresh_cards();
  place_pointer(pointer_x,pointer_y);
  while(1)
  {
    delay(8);
    check_joypad();
    
    j=0;
    for(i=0;i<16;i++) if(tableau[i]==0) j++;
    if(j==0) discard_active=FALSE;
    if(discard==0 && deck[0]!=0 && discard_active)
    {
      move_deck_discard();
      refresh_cards();
    }

    if(deck[0]==0 
      && discard==0 
      && tableau[5]==0 
      && tableau[6]==0 
      && tableau[9]==0 
      && tableau[10]==0) win_animation();
  }
}